@@ -1,9 +1,9 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: CL-PPCRE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/cl-ppcre/api.lisp,v 1.75 2007/09/13 07:52:13 edi Exp $
+;;; $Header: /usr/local/cvsrep/cl-ppcre/api.lisp,v 1.84 2008/07/06 18:12:04 edi Exp $
 
 ;;; The external API for creating and using scanners.
 
-;;; Copyright (c) 2002-2007, Dr. Edmund Weitz. All rights reserved.
+;;; Copyright (c) 2002-2008, Dr. Edmund Weitz. All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -29,7 +29,7 @@
 ;;; NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;; SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-(in-package #:cl-ppcre)
+(in-package :cl-ppcre)
 
 (defgeneric create-scanner (regex &key case-insensitive-mode
                                        multi-line-mode
@@ -39,10 +39,10 @@
   (:documentation "Accepts a regular expression - either as a
 parse-tree or as a string - and returns a scan closure which will scan
 strings for this regular expression and a list mapping registers to
-their names \(NIL stands for unnamed ones). The \"mode\" keyboard
-arguments are equivalent to the imsx modifiers in Perl. If DESTRUCTIVE
-is not NIL the function is allowed to destructively modify its first
-argument \(but only if it's a parse tree)."))
+their names \(NIL stands for unnamed ones).  The \"mode\" keyboard
+arguments are equivalent to the imsx modifiers in Perl.  If
+DESTRUCTIVE is not NIL, the function is allowed to destructively
+modify its first argument \(but only if it's a parse tree)."))
 
 #-:use-acl-regexp2-engine
 (defmethod create-scanner ((regex-string string) &key case-insensitive-mode
@@ -76,8 +76,7 @@ argument \(but only if it's a parse tree)."))
   (declare #.*standard-optimize-settings*)
   (declare (ignore destructive))
   (when (or case-insensitive-mode multi-line-mode single-line-mode extended-mode)
-    (signal-ppcre-invocation-error
-     "You can't use the keyword arguments to modify an existing scanner."))
+    (signal-invocation-error "You can't use the keyword arguments to modify an existing scanner."))
   scanner)
 
 #-:use-acl-regexp2-engine
@@ -88,8 +87,7 @@ argument \(but only if it's a parse tree)."))
                                                destructive)
   (declare #.*standard-optimize-settings*)
   (when extended-mode
-    (signal-ppcre-invocation-error
-     "Extended mode doesn't make sense in parse trees."))
+    (signal-invocation-error "Extended mode doesn't make sense in parse trees."))
   ;; convert parse-tree into internal representation REGEX and at the
   ;; same time compute the number of registers and the constant string
   ;; (or anchor) the regex starts with (if any)
@@ -180,7 +178,6 @@ argument \(but only if it's a parse tree)."))
 
 #+:use-acl-regexp2-engine
 (declaim (inline create-scanner))
-
 #+:use-acl-regexp2-engine
 (defmethod create-scanner ((scanner regexp::regular-expression) &key case-insensitive-mode
                                                                      multi-line-mode
@@ -190,8 +187,7 @@ argument \(but only if it's a parse tree)."))
   (declare #.*standard-optimize-settings*)
   (declare (ignore destructive))
   (when (or case-insensitive-mode multi-line-mode single-line-mode extended-mode)
-    (signal-ppcre-invocation-error
-     "You can't use the keyword arguments to modify an existing scanner."))
+    (signal-invocation-error "You can't use the keyword arguments to modify an existing scanner."))
   scanner)
 
 #+:use-acl-regexp2-engine
@@ -254,7 +250,6 @@ internal purposes."))
 
 #+:use-acl-regexp2-engine
 (declaim (inline scan))
-
 #+:use-acl-regexp2-engine
 (defmethod scan ((parse-tree t) target-string
                                 &key (start 0)
@@ -292,12 +287,12 @@ internal purposes."))
 (defun scan-to-strings (regex target-string &key (start 0)
                                                  (end (length target-string))
                                                  sharedp)
-  (declare #.*standard-optimize-settings*)
   "Like SCAN but returns substrings of TARGET-STRING instead of
 positions, i.e. this function returns two values on success: the whole
 match as a string plus an array of substrings (or NILs) corresponding
-to the matched registers. If SHAREDP is true, the substrings may share
-structure with TARGET-STRING."
+to the matched registers.  If SHAREDP is true, the substrings may
+share structure with TARGET-STRING."
+  (declare #.*standard-optimize-settings*)
   (multiple-value-bind (match-start match-end reg-starts reg-ends)
       (scan regex target-string :start start :end end)
     (unless match-start
@@ -329,11 +324,11 @@ structure with TARGET-STRING."
   "Executes BODY with the variables in VAR-LIST bound to the
 corresponding register groups after TARGET-STRING has been matched
 against REGEX, i.e. each variable is either bound to a string or to
-NIL. If there is no match, BODY is _not_ executed. For each element of
-VAR-LIST which is NIL there's no binding to the corresponding register
-group. The number of variables in VAR-LIST must not be greater than
-the number of register groups. If SHAREDP is true, the substrings may
-share structure with TARGET-STRING."
+NIL.  If there is no match, BODY is _not_ executed. For each element
+of VAR-LIST which is NIL there's no binding to the corresponding
+register group.  The number of variables in VAR-LIST must not be
+greater than the number of register groups.  If SHAREDP is true, the
+substrings may share structure with TARGET-STRING."
   (with-rebinding (target-string)
     (with-unique-names (match-start match-end reg-starts reg-ends
                                     start-index substr-fn)
@@ -368,61 +363,57 @@ share structure with TARGET-STRING."
                     &environment env)
   "Iterates over TARGET-STRING and tries to match REGEX as often as
 possible evaluating BODY with MATCH-START, MATCH-END, REG-STARTS, and
-REG-ENDS bound to the four return values of each match in turn. After
+REG-ENDS bound to the four return values of each match in turn.  After
 the last match, returns RESULT-FORM if provided or NIL otherwise. An
 implicit block named NIL surrounds DO-SCANS; RETURN may be used to
-terminate the loop immediately. If REGEX matches an empty string the
+terminate the loop immediately.  If REGEX matches an empty string the
 scan is continued one position behind this match. BODY may start with
 declarations."
   (with-rebinding (target-string)
-    (with-unique-names (%start %end %regex scanner loop-tag block-name)
+    (with-unique-names (%start %end %regex scanner)
       (declare (ignorable %regex scanner))
       ;; the NIL BLOCK to enable exits via (RETURN ...)
       `(block nil
-        (let* ((,%start (or ,start 0))
-               (,%end (or ,end (length ,target-string)))
-               ,@(unless (constantp regex env)
-                   ;; leave constant regular expressions as they are -
-                   ;; SCAN's compiler macro will take care of them;
-                   ;; otherwise create a scanner unless the regex is
-                   ;; already a function (otherwise SCAN will do this
-                   ;; on each iteration)
-                   `((,%regex ,regex)
-                     (,scanner (typecase ,%regex
-                                 (function ,%regex)
-                                 (t (create-scanner ,%regex)))))))
-          ;; coerce TARGET-STRING to a simple string unless it is one
-          ;; already (otherwise SCAN will do this on each iteration)
-          (setq ,target-string
-                  (maybe-coerce-to-simple-string ,target-string))
-          ;; a named BLOCK so we can exit the TAGBODY
-          (block ,block-name
-            (tagbody
-              ,loop-tag
-              ;; invoke SCAN and bind the returned values to the
-              ;; provided variables
-              (multiple-value-bind
-                    (,match-start ,match-end ,reg-starts ,reg-ends)
-                  (scan ,(cond ((constantp regex env) regex)
-                               (t scanner))
-                        ,target-string :start ,%start :end ,%end
-                                       :real-start-pos (or ,start 0))
-                ;; declare the variables to be IGNORABLE to prevent the
-                ;; compiler from issuing warnings
-                (declare
-                  (ignorable ,match-start ,match-end ,reg-starts ,reg-ends))
-                (unless ,match-start
-                  ;; stop iteration on first failure
-                  (return-from ,block-name ,result-form))
-                ;; execute BODY (wrapped in LOCALLY so it can start with
-                ;; declarations)
-                (locally
-                  ,@body)
-                ;; advance by one position if we had a zero-length match
-                (setq ,%start (if (= ,match-start ,match-end)
-                                (1+ ,match-end)
-                                ,match-end)))
-              (go ,loop-tag))))))))
+         (let* ((,%start (or ,start 0))
+                (,%end (or ,end (length ,target-string)))
+                ,@(unless (constantp regex env)
+                    ;; leave constant regular expressions as they are -
+                    ;; SCAN's compiler macro will take care of them;
+                    ;; otherwise create a scanner unless the regex is
+                    ;; already a function (otherwise SCAN will do this
+                    ;; on each iteration)
+                    `((,%regex ,regex)
+                      (,scanner (typecase ,%regex
+                                  (function ,%regex)
+                                  (t (create-scanner ,%regex)))))))
+           ;; coerce TARGET-STRING to a simple string unless it is one
+           ;; already (otherwise SCAN will do this on each iteration)
+           (setq ,target-string
+                 (maybe-coerce-to-simple-string ,target-string))
+           (loop
+            ;; invoke SCAN and bind the returned values to the
+            ;; provided variables
+            (multiple-value-bind
+                (,match-start ,match-end ,reg-starts ,reg-ends)
+                (scan ,(cond ((constantp regex env) regex)
+                             (t scanner))
+                      ,target-string :start ,%start :end ,%end
+                      :real-start-pos (or ,start 0))
+              ;; declare the variables to be IGNORABLE to prevent the
+              ;; compiler from issuing warnings
+              (declare
+               (ignorable ,match-start ,match-end ,reg-starts ,reg-ends))
+              (unless ,match-start
+                ;; stop iteration on first failure
+                (return ,result-form))
+              ;; execute BODY (wrapped in LOCALLY so it can start with
+              ;; declarations)
+              (locally
+                ,@body)
+              ;; advance by one position if we had a zero-length match
+              (setq ,%start (if (= ,match-start ,match-end)
+                              (1+ ,match-end)
+                              ,match-end)))))))))
 
 (defmacro do-matches ((match-start match-end regex
                                    target-string
@@ -431,11 +422,11 @@ declarations."
                       &body body)
   "Iterates over TARGET-STRING and tries to match REGEX as often as
 possible evaluating BODY with MATCH-START and MATCH-END bound to the
-start/end positions of each match in turn. After the last match,
-returns RESULT-FORM if provided or NIL otherwise. An implicit block
+start/end positions of each match in turn.  After the last match,
+returns RESULT-FORM if provided or NIL otherwise.  An implicit block
 named NIL surrounds DO-MATCHES; RETURN may be used to terminate the
-loop immediately. If REGEX matches an empty string the scan is
-continued one position behind this match. BODY may start with
+loop immediately.  If REGEX matches an empty string the scan is
+continued one position behind this match.  BODY may start with
 declarations."
   ;; this is a simplified form of DO-SCANS - we just provide two dummy
   ;; vars and ignore them
@@ -454,12 +445,12 @@ declarations."
                                  &body body)
   "Iterates over TARGET-STRING and tries to match REGEX as often as
 possible evaluating BODY with MATCH-VAR bound to the substring of
-TARGET-STRING corresponding to each match in turn. After the last
-match, returns RESULT-FORM if provided or NIL otherwise. An implicit
+TARGET-STRING corresponding to each match in turn.  After the last
+match, returns RESULT-FORM if provided or NIL otherwise.  An implicit
 block named NIL surrounds DO-MATCHES-AS-STRINGS; RETURN may be used to
-terminate the loop immediately. If REGEX matches an empty string the
-scan is continued one position behind this match. If SHAREDP is true,
-the substrings may share structure with TARGET-STRING. BODY may start
+terminate the loop immediately.  If REGEX matches an empty string the
+scan is continued one position behind this match.  If SHAREDP is true,
+the substrings may share structure with TARGET-STRING.  BODY may start
 with declarations."
   (with-rebinding (target-string)
     (with-unique-names (match-start match-end substr-fn)
@@ -479,15 +470,16 @@ with declarations."
   "Iterates over TARGET-STRING and tries to match REGEX as often as
 possible evaluating BODY with the variables in VAR-LIST bound to the
 corresponding register groups for each match in turn, i.e. each
-variable is either bound to a string or to NIL. For each element of
+variable is either bound to a string or to NIL.  For each element of
 VAR-LIST which is NIL there's no binding to the corresponding register
 group. The number of variables in VAR-LIST must not be greater than
-the number of register groups. After the last match, returns
-RESULT-FORM if provided or NIL otherwise. An implicit block named NIL
+the number of register groups.  After the last match, returns
+RESULT-FORM if provided or NIL otherwise.  An implicit block named NIL
 surrounds DO-REGISTER-GROUPS; RETURN may be used to terminate the loop
 immediately. If REGEX matches an empty string the scan is continued
-one position behind this match. If SHAREDP is true, the substrings may
-share structure with TARGET-STRING. BODY may start with declarations."
+one position behind this match.  If SHAREDP is true, the substrings
+may share structure with TARGET-STRING.  BODY may start with
+declarations."
   (with-rebinding (target-string)
     (with-unique-names (substr-fn match-start match-end
                                   reg-starts reg-ends start-index)
@@ -514,11 +506,11 @@ share structure with TARGET-STRING. BODY may start with declarations."
 (defun all-matches (regex target-string
                           &key (start 0)
                                (end (length target-string)))
-  (declare #.*standard-optimize-settings*)
   "Returns a list containing the start and end positions of all
 matches of REGEX against TARGET-STRING, i.e. if there are N matches
-the list contains (* 2 N) elements. If REGEX matches an empty string
+the list contains (* 2 N) elements.  If REGEX matches an empty string
 the scan is continued one position behind this match."
+  (declare #.*standard-optimize-settings*)
   (let (result-list)
     (do-matches (match-start match-end
                  regex target-string
@@ -540,11 +532,11 @@ compile time."
                                      &key (start 0)
                                           (end (length target-string))
                                           sharedp)
-  (declare #.*standard-optimize-settings*)
   "Returns a list containing all substrings of TARGET-STRING which
 match REGEX. If REGEX matches an empty string the scan is continued
 one position behind this match. If SHAREDP is true, the substrings may
 share structure with TARGET-STRING."
+  (declare #.*standard-optimize-settings*)
   (let (result-list)
     (do-matches-as-strings (match regex target-string (nreverse result-list)
                                   :start start :end end :sharedp sharedp)
@@ -567,18 +559,18 @@ compile time."
                          with-registers-p
                          omit-unmatched-p
                          sharedp)
-  (declare #.*standard-optimize-settings*)
   "Matches REGEX against TARGET-STRING as often as possible and
-returns a list of the substrings between the matches. If
+returns a list of the substrings between the matches.  If
 WITH-REGISTERS-P is true, substrings corresponding to matched
-registers are inserted into the list as well. If OMIT-UNMATCHED-P is
+registers are inserted into the list as well.  If OMIT-UNMATCHED-P is
 true, unmatched registers will simply be left out, otherwise they will
-show up as NIL. LIMIT limits the number of elements returned -
-registers aren't counted. If LIMIT is NIL (or 0 which is equivalent),
-trailing empty strings are removed from the result list.  If REGEX
-matches an empty string the scan is continued one position behind this
-match. If SHAREDP is true, the substrings may share structure with
-TARGET-STRING."
+show up as NIL.  LIMIT limits the number of elements returned -
+registers aren't counted.  If LIMIT is NIL \(or 0 which is
+equivalent), trailing empty strings are removed from the result list.
+If REGEX matches an empty string the scan is continued one position
+behind this match.  If SHAREDP is true, the substrings may share
+structure with TARGET-STRING."
+  (declare #.*standard-optimize-settings*)
   ;; initialize list of positions POS-LIST to extract substrings with
   ;; START so that the start of the next match will mark the end of
   ;; the first substring
@@ -641,13 +633,13 @@ TARGET-STRING."
 
 (defun string-case-modifier (str from to start end)
   (declare #.*standard-optimize-settings*)
-  (declare (type fixnum from to start end))
+  (declare (fixnum from to start end))
   "Checks whether all words in STR between FROM and TO are upcased,
 downcased or capitalized and returns a function which applies a
-corresponding case modification to strings. Returns #'IDENTITY
+corresponding case modification to strings.  Returns #'IDENTITY
 otherwise, especially if words in the target area extend beyond FROM
-or TO. STR is supposed to be bounded by START and END. It is assumed
-that (<= START FROM TO END)."
+or TO.  STR is supposed to be bounded by START and END.  It is assumed
+that \(<= START FROM TO END)."
   (case
       (if (or (<= to from)
               (and (< start from)
@@ -744,9 +736,8 @@ S-expression."))
                           ((#\\) :backslash)))))
           (when (and (numberp token) (< token 0))
             ;; make sure we don't accept something like "\\0"
-            (signal-ppcre-invocation-error
-             "Illegal substring ~S in replacement string"
-             (subseq replacement-string match-start match-end)))
+            (signal-invocation-error "Illegal substring ~S in replacement string."
+                                     (subseq replacement-string match-start match-end)))
           (push token collector))
         ;; remember where the match ended
         (setq from match-end))
@@ -805,9 +796,8 @@ S-expression."))
                               ((#\\) :backslash)))))
               (when (and (numberp token) (< token 0))
                 ;; make sure we don't accept something like "\\0"
-                (signal-ppcre-invocation-error
-                 "Illegal substring ~S in replacement string"
-                 (subseq replacement match-start match-end)))
+                (signal-invocation-error "Illegal substring ~S in replacement string."
+                                         (subseq replacement match-start match-end)))
               (push token collector))
             ;; remember where the match ended
             (setq from match-end))
@@ -847,9 +837,8 @@ corresponding string."
                    (when (>= token reg-bound)
                      ;; but only if the register was referenced in the
                      ;; regular expression
-                     (signal-ppcre-invocation-error
-                      "Reference to non-existent register ~A in replacement string"
-                      (1+ token)))
+                     (signal-invocation-error "Reference to non-existent register ~A in replacement string."
+                                              (1+ token)))
                    (when (svref reg-starts token)
                      ;; and only if it matched, i.e. no match results
                      ;; in an empty string
@@ -913,11 +902,11 @@ corresponding string."
 
 (defun replace-aux (target-string replacement pos-list reg-list start end
                                   preserve-case simple-calls element-type)
+  "Auxiliary function used by REGEX-REPLACE and REGEX-REPLACE-ALL.
+POS-LIST contains a list with the start and end positions of all
+matches while REG-LIST contains a list of arrays representing the
+corresponding register start and end positions."
   (declare #.*standard-optimize-settings*)
-  "Auxiliary function used by REGEX-REPLACE and
-REGEX-REPLACE-ALL. POS-LIST contains a list with the start and end
-positions of all matches while REG-LIST contains a list of arrays
-representing the corresponding register start and end positions."
   ;; build the template once before we start the loop
   (let ((replacement-template (build-replacement-template replacement)))
     (with-output-to-string (s nil :element-type element-type)
@@ -953,13 +942,12 @@ representing the corresponding register start and end positions."
               ;; no replacement
               do (write-string target-string s :start from :end to)))))
 
-(defun regex-replace (regex target-string replacement
-                            &key (start 0)
+(defun regex-replace (regex target-string replacement &key
+                            (start 0)
                             (end (length target-string))
                             preserve-case
                             simple-calls
                             (element-type #+:lispworks 'lw:simple-char #-:lispworks 'character))
-  (declare #.*standard-optimize-settings*)
   "Try to match TARGET-STRING between START and END against REGEX and
 replace the first match with REPLACEMENT.  Two values are returned;
 the modified string, and T if REGEX matched or NIL otherwise.
@@ -989,14 +977,15 @@ match. The result will always be a fresh string, even if REGEX doesn't
 match.
 
   ELEMENT-TYPE is the element type of the resulting string."
+  (declare #.*standard-optimize-settings*)
   (multiple-value-bind (match-start match-end reg-starts reg-ends)
       (scan regex target-string :start start :end end)
     (if match-start
       (values (replace-aux target-string replacement
-                   (list match-start match-end)
-                   (list reg-starts reg-ends)
-                   start end preserve-case
-                   simple-calls element-type)
+                           (list match-start match-end)
+                           (list reg-starts reg-ends)
+                           start end preserve-case
+                           simple-calls element-type)
               t)
       (values (subseq target-string start end)
               nil))))
@@ -1010,13 +999,12 @@ match.
                           ,target-string ,replacement ,@rest))
         (t form)))
 
-(defun regex-replace-all (regex target-string replacement
-                                &key (start 0)
-                                     (end (length target-string))
-                                     preserve-case
-                                     simple-calls
-                                     (element-type #+:lispworks 'lw:simple-char #-:lispworks 'character))
-  (declare #.*standard-optimize-settings*)
+(defun regex-replace-all (regex target-string replacement &key
+                                (start 0)
+                                (end (length target-string))
+                                preserve-case
+                                simple-calls
+                                (element-type #+:lispworks 'lw:simple-char #-:lispworks 'character))
   "Try to match TARGET-STRING between START and END against REGEX and
 replace all matches with REPLACEMENT.  Two values are returned; the
 modified string, and T if REGEX matched or NIL otherwise.
@@ -1046,21 +1034,22 @@ match. The result will always be a fresh string, even if REGEX doesn't
 match.
 
   ELEMENT-TYPE is the element type of the resulting string."
+  (declare #.*standard-optimize-settings*)
   (let ((pos-list '())
         (reg-list '()))
     (do-scans (match-start match-end reg-starts reg-ends regex target-string
-               nil
-               :start start :end end)
+                           nil
+                           :start start :end end)
       (push match-start pos-list)
       (push match-end pos-list)
       (push reg-starts reg-list)
       (push reg-ends reg-list))
     (if pos-list
       (values (replace-aux target-string replacement
-                   (nreverse pos-list)
-                   (nreverse reg-list)
-                   start end preserve-case
-                   simple-calls element-type)
+                           (nreverse pos-list)
+                           (nreverse reg-list)
+                           start end preserve-case
+                           simple-calls element-type)
               t)
       (values (subseq target-string start end)
               nil))))
@@ -1083,59 +1072,68 @@ which matches REGEX. Optionally evaluates and returns RETURN-FORM at
 the end. If CASE-INSENSITIVE is true and REGEX isn't already a
 scanner, a case-insensitive scanner is used."
   (with-rebinding (regex)
-    (with-unique-names (scanner %packages next morep)
+    (with-unique-names (scanner %packages next morep hash)
       `(let* ((,scanner (create-scanner ,regex
                                         :case-insensitive-mode
                                         (and ,case-insensitive
                                              (not (functionp ,regex)))))
               (,%packages (or ,packages
-                              (list-all-packages))))
+                              (list-all-packages)))
+              (,hash (make-hash-table :test #'eq)))
          (with-package-iterator (,next ,%packages :external :internal :inherited)
            (loop
              (multiple-value-bind (,morep symbol)
                  (,next)
                (unless ,morep
                  (return ,return-form))
-               (when (scan ,scanner (symbol-name symbol))
-                 ,@body))))))))
+               (unless (gethash symbol ,hash)
+                 (when (scan ,scanner (symbol-name symbol))
+                   (setf (gethash symbol ,hash) t)
+                   ,@body)))))))))
 
 ;;; The following two functions were provided by Karsten Poeck
 
 #+:cormanlisp
-(defmacro do-with-all-symbols ((variable package-packagelist) &body body)
-  (with-unique-names (pack-var iter-sym)
-    `(if (listp ,package-packagelist)
-      (dolist (,pack-var ,package-packagelist)
-        (do-symbols (,iter-sym ,pack-var)
+(defmacro do-with-all-symbols ((variable package-or-packagelist) &body body)
+  "Executes BODY with VARIABLE bound to each symbol in
+PACKAGE-OR-PACKAGELIST \(a designator for a list of packages) in
+turn."
+  (with-unique-names (pack-var)
+    `(if (listp ,package-or-packagelist)
+      (dolist (,pack-var ,package-or-packagelist)
+        (do-symbols (,variable ,pack-var)
           ,@body))
-      (do-symbols (,iter-sym ,package-packagelist)
+      (do-symbols (,variable ,package-or-packagelist)
         ,@body))))
 
 #+:cormanlisp
 (defmacro regex-apropos-aux ((regex packages case-insensitive &optional return-form)
                              &body body)
-  "Auxiliary macro used by REGEX-APROPOS and REGEX-APROPOS-LIST. Loops
-through PACKAGES and executes BODY with SYMBOL bound to each symbol
-which matches REGEX. Optionally evaluates and returns RETURN-FORM at
-the end. If CASE-INSENSITIVE is true and REGEX isn't already a
-scanner, a case-insensitive scanner is used."
+  "Auxiliary macro used by REGEX-APROPOS and REGEX-APROPOS-LIST.
+Loops through PACKAGES and executes BODY with SYMBOL bound to each
+symbol which matches REGEX.  Optionally evaluates and returns
+RETURN-FORM at the end.  If CASE-INSENSITIVE is true and REGEX isn't
+already a scanner, a case-insensitive scanner is used."
   (with-rebinding (regex)
-    (with-unique-names (scanner %packages)
+    (with-unique-names (scanner %packages hash)
       `(let* ((,scanner (create-scanner ,regex
                          :case-insensitive-mode
                          (and ,case-insensitive
                               (not (functionp ,regex)))))
               (,%packages (or ,packages
-                             (list-all-packages))))
+                             (list-all-packages)))
+              (,hash (make-hash-table :test #'eq)))
         (do-with-all-symbols (symbol ,%packages)
-          (when (scan ,scanner (symbol-name symbol))
-            ,@body))
+          (unless (gethash symbol ,hash)
+            (when (scan ,scanner (symbol-name symbol))
+              (setf (gethash symbol ,hash) t)
+              ,@body)))
         ,return-form))))
 
 (defun regex-apropos-list (regex &optional packages &key (case-insensitive t))
   (declare #.*standard-optimize-settings*)
   "Similar to the standard function APROPOS-LIST but returns a list of
-all symbols which match the regular expression REGEX. If
+all symbols which match the regular expression REGEX.  If
 CASE-INSENSITIVE is true and REGEX isn't already a scanner, a
 case-insensitive scanner is used."
   (let ((collector '()))
@@ -1183,11 +1181,11 @@ meaningful information about a symbol."
     (condition ()
       ;; this seems to be necessary due to some errors I encountered
       ;; with LispWorks
-      (format t "~&~S [an error occured while trying to print more info]" symbol))))
+      (format t "~&~S [an error occurred while trying to print more info]" symbol))))
 
 (defun regex-apropos (regex &optional packages &key (case-insensitive t))
   "Similar to the standard function APROPOS but returns a list of all
-symbols which match the regular expression REGEX. If CASE-INSENSITIVE
+symbols which match the regular expression REGEX.  If CASE-INSENSITIVE
 is true and REGEX isn't already a scanner, a case-insensitive scanner
 is used."
   (declare #.*standard-optimize-settings*)
@@ -1230,7 +1228,7 @@ sections. These sections may nest."
        (quote-token-replace-scanner "\\\\([QE])"))
   (defun clean-comments (string &optional extended-mode)
     "Clean \(?#...) comments within STRING for quoting, i.e. convert
-\\Q to Q and \\E to E. If EXTENDED-MODE is true, also clean
+\\Q to Q and \\E to E.  If EXTENDED-MODE is true, also clean
 end-of-line comments, i.e. those starting with #\\# and ending with
 #\\Newline."
     (flet ((remove-tokens (target-string start end match-start
@@ -1249,7 +1247,7 @@ end-of-line comments, i.e. those starting with #\\# and ending with
                          #'remove-tokens))))
 
 (defun parse-tree-synonym (symbol)
-  "Returns the parse tree the SYMBOL symbol is a synonym for. Returns
+  "Returns the parse tree the SYMBOL symbol is a synonym for.  Returns
 NIL is SYMBOL wasn't yet defined to be a synonym."
   (get symbol 'parse-tree-synonym))
 
@@ -1259,6 +1257,6 @@ NIL is SYMBOL wasn't yet defined to be a synonym."
 
 (defmacro define-parse-tree-synonym (name parse-tree)
   "Defines the symbol NAME to be a synonym for the parse tree
-PARSE-TREE. Both arguments are quoted."
+PARSE-TREE.  Both arguments are quoted."
   `(eval-when (:compile-toplevel :load-toplevel :execute)
      (setf (parse-tree-synonym ',name) ',parse-tree)))
